from collective.lineage.interfaces import IChildSite
from plone.dexterity.interfaces import IDexterityContent
from plone.memoize import ram
from zope.interface import provider
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

import plone.api


# cache until next zope restart
# XXX: ideally invalidate when new subsites are created
@ram.cache(lambda *args: ())
@provider(IVocabularyFactory)
def childSiteVocabulary(context):
    """returns the available childsites of the portal"""
    portal = plone.api.portal.get()
    if IDexterityContent.providedBy(portal):
        terms = [SimpleTerm(value=portal.UID(), token=portal.UID(), title="Root Portal")]
    else:
        terms = []
    brains = plone.api.content.find(
        object_provides=IChildSite.__identifier__,
        path=portal.getPhysicalPath(),
        sort_on="sortable_title",
    )
    terms += [
        SimpleTerm(value=brain.UID, token=brain.UID, title=brain.Title)
        for brain in brains
    ]
    return SimpleVocabulary(terms)



from Acquisition import aq_parent
from plone.memoize.view import memoize_contextless
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory

import plone.api


# Special cases
try:
    from plone.event.interfaces import IOccurrence
except ImportError:
    IOccurrence = None


class ChildsiteView(BrowserView):
    @memoize_contextless
    def titleForKey(self, key):
        """Returns the childsite's title for their vocabulary value."""
        util = getUtility(IVocabularyFactory, "lineage.childsites")
        vocab = util(self.context)
        try:
            return vocab.getTerm(key).title
        except LookupError:
            return key

    def childsiteForContext(self, item):
        """Returns the childsite UUID for a context object by looking up it's
        brain in the catalog.
        """
        childsite = ""
        if IOccurrence and IOccurrence.providedBy(item):
            item = aq_parent(item)
        portal = plone.api.portal.get()
        cat = getToolByName(self.context, "portal_catalog")
        res = cat(UID=IUUID(item, None), path=portal.getPhysicalPath())
        if res:
            childsite = res[0].childsite
        return childsite



from Products.CMFCore.utils import getToolByName

import logging


logger = logging.getLogger("lineage.index")


def upgrade_uuid(context):
    """Reindex the childsite index to use UUID instead of id values. This makes
    it possible to have multiple childsites with the same id in different
    paths.
    """
    index_id = "childsite"
    cat = getToolByName(context, "portal_catalog")
    cat.delColumn(index_id)
    cat.addColumn(index_id)
    res = (it.getObject() for it in cat.searchResults())  # generator expr.
    for cnt, ob in enumerate(res):
        ob.reindexObject(idxs=(index_id,))
        if cnt % 100 == 0:
            # 100-batch
            logger.info(f"Reindex next batch, starting with {cnt}")



from setuptools import find_packages
from setuptools import setup


version = "1.0"
short_description = (
    "Adds an index and metadata that allows to search for "
    "content of a collective.lineage childsite"
)
long_description = "\n\n".join(
    [
        open("README.rst").read(),
        open("CHANGES.rst").read(),
    ]
)


setup(
    name="lineage.index",
    version=version,
    description=short_description,
    long_description=long_description,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: Addon",
        "Framework :: Plone :: 5.2",
        "Framework :: Plone :: 6.0",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",

    ],
    keywords="",
    author="Harald Friessnegger",
    author_email="harald@webmeisterei.com",
    url="http://github/com/collective/lineage.index",
    license="GPL",
    packages=find_packages("src"),
    package_dir={"": "src"},
    namespace_packages=["lineage"],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "setuptools",
        "collective.lineage",
        "plone.api",
    ],
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



from Acquisition import aq_base
from collective.lineage.interfaces import IChildSite
from plone.dexterity.interfaces import IDexterityContent
from plone.indexer.decorator import indexer
from plone.uuid.interfaces import IUUID
from Products.CMFCore.interfaces import IContentish
from Products.CMFPlone import utils

import plone.api


def getNextChildSite(context, portal):
    """Returns the nearest parent object implementing IChildSite.
    Code borrowed from plone.app.layout.navigation.root.getNavigationRootObject
    """
    obj = context
    while not IChildSite.providedBy(obj) and aq_base(obj) is not aq_base(portal):
        obj = utils.parent(obj)
        if obj is None:
            return
    return obj


@indexer(IContentish)
def childsite(obj):
    """Return the uuid of the closest childsite up the hierarchy or None if
    there is no subsite.
    """
    portal = plone.api.portal.get()
    childsite = getNextChildSite(obj, portal)

    if childsite == portal:
        if not IDexterityContent.providedBy(portal):
            # in Plone 5 it's been always None, lets keep it.
            return
        return IUUID(portal)

    if childsite is None:
        raise AttributeError("no childsite found")

    return IUUID(childsite)
